
  ifndef __I2CMSSP_SERVICES
#define __I2CMSSP_SERVICES 1
  endif

; ----------------------------------------------------------------------
; FUNCOES DE CONTROLE PARA DISPOSITIVOS I2C                   06/01/2015
;
; STATUS: FUNCIONA
;
; Antes de usar as funcoes desta biblioteca, o modulo MSSP
; deve ser inicializado no modo I2C com a macro 'I2C_CONFIG'.
; Exemplo:
;           I2C_CONFIG  .20000000, .100000, SLEWRATE
;
; Definir no programa os simbolos e registadores abaixo:
; i2c_buff      RES     1
; i2c_timeoutL  RES     1
; i2c_timeoutH  RES     1
;
; NOTAS
;
; 1 - Dispositivos I2C podem ter comportamento especifico, por isso, antes
;     de usar este servico, consulte o manual do dispositivo. Um exemplo de
;     uso deste servico pode ser visto no arquivo "ds1307mssp_services.inc".
;
; 2 - A funcao i2c_start, por default, chama a funcao i2c_waitMSSP para aguardar
;     a resposta do dispositivo I2C polando o bit PIR1<SSPIF>. Para desativar 
;     essa opcao, defina no programa o simbolo 'I2C_NOWAIT_SSPIF'.
;     #define  I2C_NOWAIT_SSPIF  1
;
; 3 - TIMEOUT para algumas funcoes:
;     As funcoes i2c_start, i2c_stop, i2c_put e i2c_get usam um contador para
;     limitar o loop de espera do dispositivo I2C apos um comando ser enviado a ele.
;     Para desativar essa opcao, defina no programa o simbolo 'I2C_NO_TIMEOUT'.
;     #define I2C_NO_TIMEOUT  1
;
; ----------------------------------------------------------------------
;
; ----------------------------------------------------------------------
; MACRO: I2C_CONFIG
;
; Inicializa o modulo MSSP no modo I2C:
;
;      I2C_CONFIG  fosc, bitrate, slewrate
; fosc .................. frequencia de operacao em Hertz
; bitrate ............... taxa de transmissao do I2C em Hertz
; slewrate .............. SLEWRATE -----> com controle de slewrate
;                         NO_SLEWRATE --> sem controle de slewrate
; 
; Exemplo com fosc=20MHZ, bitrate=100KHZ com controle de slewrate:
;               I2C_CONFIG  .20000000, .100000, SLEWRATE
;
; Definir os registradores abaixo no programa:
; i2c_buff      RES   1
; i2c_timeoutH  RES   1           ; time out counter High byte
; i2c_timeoutL  RES   1           ; time out counter Low  byte
;
; ----------------------------------------------------------------------
I2C_CONFIG     MACRO   fosc, bitrate, slewrate
               ifndef SLEWRATE
SLEWRATE       equ     1
               endif
               ifndef NO_SLEWRATE
NO_SLEWRATE    equ     2
               endif

               ifndef _i2c_baudrate
                 variable    _i2c_baudrate
               endif

; ---------------------------------------------------------------------------
; O calculo do valor a ser carregado em SSPADD segue a formula do data sheet:
;                     Fosc                             Fosc - (4 * bitrate)  
;    bitrate =  ----------------     ONDE    SSPADD =  ------------------
;                4*(SSPADD + 1)                            (4 * bitrate)
; ---------------------------------------------------------------------------
;_i2c_baudrate set   ((fosc/bitrate)/4) - 1

_i2c_baudrate set   (fosc - (.4*bitrate))/(.4*bitrate)

; ---------------------------------------------------------------------------
; CONFIGURA I/O DOS PINOS I2C                                      02/01/2020
; ---------------------------------------------------------------------------
      IFDEF __18F2550
               bsf     TRISB,0                   ; SDA is input
               bsf     TRISB,1                   ; SCL is intput
      ENDIF

      IFDEF __18F4550
               bsf     TRISB,0                   ; SDA is input
               bsf     TRISB,1                   ; SCL is intput
      ENDIF

      IFDEF __18F452
               bsf     TRISC,3                   ; SCL is input
               bsf     TRISC,4                   ; SDA is intput
      ENDIF

      IFDEF __16F877A
               bsf     TRISC,3                   ; SCL is input
               bsf     TRISC,4                   ; SDA is intput
      ENDIF

      IFDEF __18F4520
               bsf     TRISC,3                   ; SCL is input
               bsf     TRISC,4                   ; SDA is intput
      ENDIF

      IFDEF __18F4620
               bsf     TRISC,3                   ; SCL is input
               bsf     TRISC,4                   ; SDA is intput
      ENDIF
; ---------------------------------------------------------------------------

               movlw   b'00111000'               ; bit 5   - Enable MSSP
                                                 ; bit 4   - Release Clock
                                                 ; bit 3:0 - I2C Master Mode 
               movwf   SSPCON1                   ; estabelece modo I2C
               movlw   _i2c_baudrate             ; pega BAUDRATE
               movwf   SSPADD                    ; especifica o BAUDRATE

          if(slewrate==SLEWRATE)
              bcf     SSPSTAT,SMP                ; Slew Rate enable for 400KHZ
          else
              bsf     SSPSTAT,SMP                ; Slew Rate disable for 100KHZ and 1MHZ 
          endif
               ENDM


I2C_SHUTDOWN   MACRO
               clrf   SSPCON1
               ENDM

; ----------------------------------------------------------------------
; Funcao: i2c_wait_idle
;
; Espera o bus I2c ficar livre.
;
; Entrada: nao ha
;
; Saida  : W=0 --> OK
; ----------------------------------------------------------------------
i2cIdle
i2c_idle       movf    SSPCON2, w                ; pega SSPCON2
               andlw   0x1F                      ; check ACKEN/RCEN/PEN/RSEN/SEN
               bnz     i2c_idle                  ; espera...
i2c_idle1      btfsc   SSPSTAT, 2                ; transmissao em andamento???
               goto    i2c_idle1                 ; SIM --> espera terminar
               retlw   .0                        ; NAO --> fim

; ------------------------------------------------------------------------
; Aguarda um ACK do slave
; ------------------------------------------------------------------------
i2cWaitAck
i2c_waitACK    call    i2c_waitMSSP              ; aguarda SSPIF ser ligado
i2c_waitACK1   btfsc   SSPCON2,ACKSTAT           ; espera DS1307 responder com ACK
               goto    i2c_waitACK1              ; fica esperando ACK
               return

; ------------------------------------------------------------------------
; Aguarda uma interrupcao do modulo MSSP
; ------------------------------------------------------------------------
i2cWaitMssp
i2c_waitMSSP
  ifdef I2C_NOWAIT_SSPIF
     messg "I2CMSSP SERVICES - MSSP WAIT SSPIF NAO USADO"
  else
               btfss   PIR1,SSPIF                ; 
               goto    i2c_waitMSSP              ; 
               bcf     PIR1,SSPIF                ; 
  endif
               return                            ; 

; ----------------------------------------------------------------------
; Funcao: i2c_start
;
; Envia condicao de Start ao Slave e espera completar durante 32768
; incursoes (i2c_timeout=32768) o que representa, no minimo, cerca de 
; 6,5ms num microcontrolador rondando a 20MHZ ou 32,7ms a 4MHZ.
;
; Entrada: nao ha
;
; Saida  : W=0x00 --> ok
;          W=0x01 --> timeout
;
; ----------------------------------------------------------------------
i2cStart
i2c_start      
i2c_restart    call    i2c_idle                  ; verifica se o bus I2C esta livre
               clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               bsf     SSPCON2, SEN              ; gera condicao de Start
i2c_start1     btfss   SSPCON2, SEN              ; start ok???
               retlw   .0                        ; SIM --> fim normal
               call    i2c_waitMSSP              ; espera interrupcao do MSSP

  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; NAO --> contagem timeout
               goto    i2c_start1                ; espera Start completar
               incfsz  i2c_timeoutH,f            ; timeout
  endif

               goto    i2c_start1                ; NAO --> continua esperando
               retlw   .1                        ; SIM --> erro 1

; ----------------------------------------------------------------------
; Funcao: i2c_stop
;
; Envia condicao de Stop ao Slave e espera completar durante 32768
; incursoes (i2c_timeout=32768) o que representa, no minimo, cerca de 
; 6,5ms num microcontrolador rondando a 20MHZ ou 32,7ms a 4MHZ.
; incursoes (i2c_timeout=1024).
;
; Entrada: nao ha
;
; Saida  : W=0x00 --> ok
;          W=0x02 --> timeout
;
; ----------------------------------------------------------------------
i2cStop
i2c_stop       clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               call    i2c_idle                  ; verifica se o bus I2C esta livre
               bsf     SSPCON2, PEN              ; gera condicao de Stop
               call    i2c_waitMSSP              ; espera interrupcao do MSSP
i2c_stop1      btfss   SSPCON2, PEN              ; start ok???
               retlw   .0                        ; SIM --> fim normal
  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; NAO --> contagem timeout
               goto    i2c_stop1                 ; espera Start completar
               incfsz  i2c_timeoutH,f            ; timeout
  endif
               goto    i2c_stop1                 ; NAO --> continua esperando
               retlw   .2                        ; SIM --> erro 2


; ----------------------------------------------------------------------
; Funcao: i2c_write
;
; Envia um byte ao Slave.
;
; Entrada: W contem o valor a ser enviado.
;
; Saida  : W=0 --> ok
;          W=4 --> timeout esperando o buffer esvaziar
;          W=6 --> timeout esperando completar o envio
; ----------------------------------------------------------------------
i2cPut
i2cPutByte
i2c_put        clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_put1       btfss   SSPSTAT, BF               ; SSPBUF cheio???
               goto    i2c_put2                  ; NAO --> vai enviar
  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; SIM --> contagem timeout
               goto    i2c_put1                  ; espera completar
               incfsz  i2c_timeoutH,f            ; timeout
  endif
               goto    i2c_put1                  ; Nao --> continua esperando
               retlw   .4                        ; SIM --> erro 4 (timeout)
i2c_put2       clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               movwf   SSPBUF                    ; envia o dado
i2c_put3       btfss   SSPSTAT, BF               ; SSPBUF vazio???
               retlw   .0                        ; SIM --> fim ok
  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; contagem timeout
               goto    i2c_put3                  ; espera completar
               incfsz  i2c_timeoutH,f            ; timeout
  endif
               goto    i2c_put3                  ; Nao --> continua esperando
               retlw   .6                        ; SIM --> erro 6  (timeout)



; ----------------------------------------------------------------------
; Funcao: i2c_read
;
; Recebe um byte do Slave.
;
; Entrada: nao ha
;
; Saida  : i2c_buff contera o dado lido
;          W=0  --> ok
;          W=8  --> timeout esperando o buffer ficar pronto
;          W=10 --> timeout na leitura
;
; ----------------------------------------------------------------------
i2cGet
i2c_get        clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_get1       btfss   SSPSTAT, BF               ; SSPBUF cheio???
               goto    i2c_get2                  ; NAO --> vai receber
  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; timeout???
               goto    i2c_get1                  ; NAO --> espera completar
               incfsz  i2c_timeoutH,f            ; timeout???
  endif
               goto    i2c_get1                  ; NAO --> continua esperando
               retlw   .8                        ; SIM --> erro 8

i2c_get2       bsf     SSPCON2, RCEN             ; modo de recepcao
               clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_get3       btfsc   SSPSTAT, BF               ; SSPBUF vazio???
               goto    i2c_get4                  ; SIM --> espera SSPBUF encher 
  ifndef I2C_NO_TIMEOUT
               incfsz  i2c_timeoutL,f            ; contagem timeout
               goto    i2c_get3                  ; espera completar
               incfsz  i2c_timeoutH,f            ; timeout
  endif
               goto    i2c_get3                  ; Nao --> continua esperando
               retlw   .10                       ; SIM --> erro 10

i2c_get4       movf    SSPBUF, w                 ; pega o dado que veio em SSPBUF  
               movwf   i2c_buff                  ; salva o dado recebido
               retlw   .0                        ; fim normal

; ----------------------------------------------------------------------
; Funcao: i2c_nack
;
; Envia um NACK ao Slave
;
; Entrada: nao ha
;
; Saida  : W=0 --> ok
;
; ----------------------------------------------------------------------
i2cNack
i2c_nack       bsf     SSPCON2, ACKDT
               bsf     SSPCON2, ACKEN
i2c_nack1      btfsc   SSPCON2, ACKEN
               goto    i2c_nack1
               retlw   .0

; ----------------------------------------------------------------------
; Funcao: i2c_ack
;
; Envia um ACK ao Slave
;
; Entrada: nao ha
;
; Saida  : W=0 --> ok
;
; ----------------------------------------------------------------------
i2cAck
i2c_ack        call    i2c_idle                  ;  verifica se o bus I2C esta livre
               bcf     SSPCON2, ACKDT
               bsf     SSPCON2, ACKEN
i2c_ack1       btfsc   SSPCON2, ACKEN
               goto    i2c_ack1
               call    i2c_waitMSSP              ; 
               retlw   .0

; ----------------------------------------------------------------------
; Funcao: i2cGetWriteAddress                                  06/12/2019
; 
; Dado o endereco real de 7 bits do disposotivo I2C, esta funcao retorna
; o endereco de WRITE de 8 bits.
;
; Entrada: W contem o endereco real de 7 bits do dispositivo I2C.
;
; Saida  : W contera o endereco de WRITE de 8 bits.
;
; ----------------------------------------------------------------------
i2cGetWriteAddress
           movwf   i2c_buff  
           rlcf    i2c_buff, f
           movf    i2c_buff, w
           andlw   0xFE
           return

; ----------------------------------------------------------------------
; Funcao: i2cGetReadAddress                                   06/12/2019
; 
; Dado o endereco real de 7 bits do disposotivo I2C, esta funcao retorna
; o endereco de READ de 8 bits.
;
; Entrada: W contem o endereco real de 7 bits do dispositivo I2C.
;
; Saida  : W contera o endereco de READ de 8 bits.
;
; ----------------------------------------------------------------------
i2cGetReadAddress
           movwf   i2c_buff  
           rlcf    i2c_buff, f
           movf    i2c_buff, w
           iorlw   0x01
           return