; -----------------------------------------------------------------------------------
; SERVICO LCDI2C_SERVICES                                            DEZEMBRO DE 2019
;
; Este servico fornece Macro Instrucoes e Funcoes em linguagem Assembly para operacao
; do LCD HD44780 com driver PCF8574T.
;
; O acesso ao dispositivo I2C (PCF8574T) podera ser feito por microcontroladores PIC 
; que possuam modulo MSSP que possam ser configurados como Master em modo I2C.
;
; O chip PCF8574T faz interface entre os pinos I2C do PIC (SCL e SDA) e os pinos do 
; LCD conforme o esquema de conexoes abaixo:   
; 
; PIC                  PCF8574T        LCD1602   
; ----                 -------------   ---------------
; SCL ---------------> (15) SDA        
; SDA ---------------> (14) SCL
;                      (04) P0 ------> (04) RS
;                      (05) P1 ------> (05) RW  
;                      (06) P2 ------> (06) EN  
;                      (07) P3 ------> (03) VE (contraste)  
;                      (09) P4 ------> (11) D4   
;                      (10) P5 ------> (12) D5  
;                      (11) P6 ------> (13) D6
;                      (12) P7 ------> (14) D7
;
; NOTAS:
; 1 - O endereco I2C de 7 bits do chip PCF8574T e' fixado na fabrica em 0x27 mas outros modelos podem
;     ter enderecos que variam entre 0x20 e 0x27. O endereco de WRITE de 8 bits utilizado internamente
;     sera 0x4E. O endereco real (0x27) deve ser informado no parametro 'device' da macro LCDI2C_CONFIG.
;
; 2 - O pino P3 do PCF8574T esta conectado internamente a base de um trasistor NPN e ao pino 3 (VE)
;     do LCD. Este pino e' usado para acender/apagar o back-light do LCD atraves do software onde
;     um nivel alto acende o back-light e um nivel baixo apaga.
;
; 3 - Usar resistores de 4K7 no Bus I2C.
;
; 4 - O LCD funcionara em modo de 4 bits usando os pinos D4 a D7 para receber dados que serao 
;     enviados pelo driver em duas etapas pela funcao 'lcdSendByte': 
;     primeiro o MSB depois o LSB.
;
; 5 - As funcoes de acesso basico ao dispositivo I2C (PCF8574T) serao fornecidas pelo modulo 
;     'i2cmssp_services.inc'.
;     
; 6 - Este driver foi testado e funcionou perfeitamente em microcontroladores PIC18F4520, 18F4560 e
;     18F4550 usando clocks de 4MHZ e 20MHZ.
;
; 7 - Este servico usa os servicos 'delay_services.inc' e 'i2cmssp_services.inc'.
;
; -----------------------------------------------------------------------------------


; -----------------------------------------------------------------------------------
; Incluir os simbolos e registradores no programa:      
; -----------------------------------------------------------------------------------
;_FOSC      equ     .20000000           ; Fosc em HERTZ
;_100KHZ    equ     .100000             ; taxa de transmissao do I2C em HERTZ
;lcdi2cr1      res     .1               ; work reg
;lcdi2cr2      res     .1               ; work reg
;lcdaddress    res     .1               ; endereco do dispositivo
;lcdwradd      res     .1               ; endereco de write do dispositivo
;lcdcmd        res     .1               ; buffer de comando
;lcdbuf        res     .1               ; buffer generico
;lcdstrtime    res     .1               ; tempo de espera
;lcddisplay    res     .1               ; ultimo estado do display
;lcdflags      res     .1               ; flags do servico
;lightOn       equ     .7               ; 0->bck light apagado    1->bck light aceso
;i2c_buff      res     .1               ; para o modulo i2cmssp
;i2c_timeoutL  res     .1               ; para o modulo i2cmss
;i2c_timeoutH  res     .1               ; para o modulo i2cmssp
;temp_temp     res     .1               ; para o modulo delays
;delay_temp    res     .1               ; para o modulo delays
; -----------------------------------------------------------------------------------



; -----------------------------------------------------------------------------------
; MACRO INSTRUCOES
;
; Para configurar o LCD, mover o cursor e facilitar o envio de caracteres atraves do
; Bus I2C, algumas macro instrucoes sao desenvolvidas.  
;
; -----------------------------------------------------------------------------------
;
;
; ===> LCDI2C_CONFIG device, linhas, colunas, modo, matriz, curmode, curtype
; Onde:
; device ................ endereco I2C do LCD (default 0x27)
; linhas ................ numero de linhas do LCD
; colunas ............... numero de colunas do LCD
; modo .................. modo de operacao: 4 --> 4 bits ou 8 --> 8 bits 
; matriz ................ dimensao da matriz de caracteres: 7 --> 5x7  ou 10 --> 5x10
; curmode ............... modo do cursor: CURSOR_ON ou CURSOR_OFF
; curtype ............... cursor piscante --> BLINK  ou  nao piscante --> NOBLINK 
;
;
;
; ===> SETI2C_CURSOR linha, coluna, tipo
; Onde:
; linha ................. numero da linha
; coluna ................ numero da coluna
; tipo .................. "i" - linha e coluna inlines na macro
;                         "r" - linha e colunas em registradores
;
;
;
; ===> LCDI2C_WRITE "string"
; Onde:
; "string" .............. e' um conjunto de caracteres entre aspas
;
;
;
; ===> LCDI2C_PUT ............ Esta macro envia o caracter contido no registrador W             
;
;
; 
; ===> LCDI2C_PUTCHAR "c"
; Onde:
; "c" ................... e' um caracter entre aspas
;
;
;
; ===> LCDI2C_PRINT string_address, len
; Onde:
; string ................ e' o endereco da string na RAM
; len ................... tamanho da string
;
; -----------------------------------------------------------------------------------




; -----------------------------------------------------------------------------------
; FUNCOES DE ALTO NIVEL:
;
; Essas funcoes sao usadas na programacao normal para modificar certas
; caracteristicas e modo de operacao do LCD. As funcoes abaixo devem ser chamadas 
; com a instrucao CALL:
; -----------------------------------------------------------------------------------
; lcdCursorHome ......... Move o cursor para a posicao (1,1).
; lcdCursorBlink ........ Torna o cursor piscante.
; lcdCursorNoBlink ...... Torna o cursor estatico.
; lcdCursorOn ........... Torna o cursor visivel.
; lcdCursorOff .......... Torna o cursor invisivel.
; lcdClear .............. Apaga o LCD.
; lcdBackOn ............. Acende o Back-Light do LCD.
; lcdBackOff ............ Apaga o Back-Light do LCD.
; lcdDisplayOn .......... Habilita o LCD.
; lcdDisplayOff.......... Desabilita o LCD.
; lcdSetCgram ........... Estabelece endereco na CGRAM.
; lcdPrint .............. Envia 'ene' caracteres de uma string na RAM.
; lcdShiftRigth ......... Desloca o display N posicoes para a direita.
; lcdShiftLeft .......... Desloca o display N posicoes para a esquerda.
; -----------------------------------------------------------------------------------



; -----------------------------------------------------------------------------------
; FUNCOES DE BAIXO NIVEL USADAS PELAS FUNCOES DE ALTO NIVEL:
;
; Essas funcoes sao usadas pelas macros e funcoes de alto nivel para poder controlar
; as caracteristicas e o modo de operacao do LCD.
; Essas funcoes, via de regra, nao devem ser usadas na programacao normal.
; As funcoes abaixo devem ser chamadas com a instrucao CALL:
; -----------------------------------------------------------------------------------
; lcdOpen ............... Envia condicao de START e o endereco do dispositivo I2C.
; lcdClose .............. Envia condicao de STOP.
; lcdSendChar ........... Envia um caracter ao LCD.
; lcdSendCmd ............ Envia um comando ao LCD.
; lcdSendByte ........... Envia caracter ou comando ao LCD em duas fases: MSB e LSB.
; -----------------------------------------------------------------------------------



; -----------------------------------------------------------------------------------
; EXEMPLO DE PROGRAMA
; -----------------------------------------------------------------------------------
;
;_FOSC      equ     .20000000           ; Fosc 
;_100KHZ    equ     .100000             ; taxa de transmissao do I2C 100KHz
;#define    LIGHT_ON   iorlw   0x08     ; para acender back-light
;#define    LIGHT_OFF  andlw   0xF7     ; para apagar back-light
;lcdi2cr1      res     .1
;lcdi2cr2      res     .1
;lcdaddress    res     .1
;lcdwradd      res     .1
;lcdcmd        res     .1
;lcdbuf        res     .1
;lcdstrtime    res     .1
;lcddisplay    res     .1
;lcdflags      res     .1      ; flags 
;lightOn       equ     .7      ; 0->bck light apagado    1->bck light aceso
;i2c_buff      res     .1
;i2c_timeoutL  res     .1
;i2c_timeoutH  res     .1
;
;           #include "lcdi2c_services.inc"
;
;           LCDI2C_CONFIG  0x27, .2, .16, .4, .7, CURSOR_ON, NOBLINK
;           call   lcdClear
;           SETI2C_CURSOR .1, .1, "i"
;           LCDI2C_WRITE "Bom Dia!!!"
;           SETI2C_CURSOR .2, .1, "i"
;           LCDI2C_WRITE "Isto e um teste"
;           tempo...
;           call   lcdClear
;           call   lcdCursorBlink
;           LCDI2C_WRITE "CURSOR PISCANDO"
;           tempo...
;           call   lcdClear
;           LCDI2C_WRITE "CURSOR INVISIVEL"
;           call   lcdCursorNoBlink
;           call   lcdCursorOff
;           tempo...
; -----------------------------------------------------------------------------------



; -----------------------------------------------------------------------------------
; CARGA DE PRE REQUISITOS PARA O SERVICO    
; -----------------------------------------------------------------------------------
   IFNDEF __DELAY_SERVICE
       #include "Includes\delay_services.inc"
       messg "LCDI2C SERVICES - Arquivo delay_services.inc foi incluido"
   ENDIF

   IFNDEF __I2CMSSP_SERVICES
       #include "Includes\i2cmssp_services.inc"
       messg "LCDI2C SERVICES - Arquivo i2cmssp_services.inc foi incluido"
   ENDIF

; -----------------------------------------------------------------------------------
; LCDI2C_CONFIG                                                           
;
; Configuracao do LCD I2C.
; Esta macro faz a configuracao inicial do LCD I2C deixando-o pronto para receber
; dados. 
;
;       LCDI2C_CONFIG  device, linhas, colunas, modo, matriz, curmode, curtype
;
; device ................ endereco I2C do LCD (default 0x27)
; linhas ................ numero de linhas do LCD
; colunas ............... numero de colunas do LCD
; modo .................. modo de operacao: 4 --> 4 bits ou 8 --> 8 bits 
; matriz ................ dimensao da matriz de caracteres: 7 --> 5x7  ou 10 --> 5x10
; curmode ............... modo do cursor: CURSOR_ON ou CURSOR_OFF
; curtype ............... cursor piscante --> BLINK  ou  nao piscante --> NOBLINK
;
; EXEMPLO:
;
;           LCDI2C_CONFIG  0x27, .2, .16, .4, .10, CURSOR_OFF, NOBLINK
;
; -----------------------------------------------------------------------------------
LCDI2C_CONFIG MACRO device, linhas, colunas, modo, matriz, curmode, curtype

  ifndef __LCDI2C__
#define  __LCDI2C__   1

CURSOR_ON  equ       .1                ; ON
CURSOR_OFF equ       .2                ; OFF
BLINK      equ       .3                ; cursor piscante
NOBLINK    equ       .4                ; cursor nao piscante

           VARIABLE  LCDI2C_ADDRESS    ; device address
           VARIABLE  LCDI2C_MAXLINES   ; numero maximo de linhas
           VARIABLE  LCDI2C_MAXCOLS    ; numero maximo de colunas
           VARIABLE  LCDI2C_MODO       ; 4 --> modo 4 bits  8 --> modo 8 bits
           VARIABLE  LCDI2C_MATRIZ     ; dimensao da matriz de caracteres --> 7 ou 10
           VARIABLE  LCDI2C_CURSOR     ; CURSOR_ON ou CURSOR_OFF
           VARIABLE  LCDI2C_CURTYPE    ; CURSOR BLINK ou NOBLINK

           VARIABLE  LCDI2C_SIZE       
           VARIABLE  LCDI2C_LINE1
           VARIABLE  LCDI2C_LINE2
           VARIABLE  LCDI2C_LINE3
           VARIABLE  LCDI2C_LINE4

LCDI2C_ADDRESS       SET device
LCDI2C_MAXLINES      SET linhas
LCDI2C_MAXCOLS       SET colunas
LCDI2C_MODO          SET modo
LCDI2C_MATRIZ        SET matriz
LCDI2C_CURSOR        SET curmode
LCDI2C_CURTYPE       SET curtype
LCDI2C_SIZE          SET LCDI2C_MAXLINES * LCDI2C_MAXCOLS

           IF LCDI2C_SIZE == .32     ; LCD 2x16
LCDI2C_LINE1 SET     0x00            ; endereco da linha 1
LCDI2C_LINE2 SET     0x40            ; endereco da linha 2
           ENDIF

           IF LCDI2C_SIZE == .64     ; LCD 4x16
LCDI2C_LINE1 SET     0x00            ; endereco da linha 1
LCDI2C_LINE2 SET     0x40            ; endereco da linha 2
LCDI2C_LINE3 SET     0x90            ; endereco da linha 3
LCDI2C_LINE4 SET     0xD0            ; endereco da linha 4
           ENDIF

           IF LCDI2C_SIZE == .80     ; LCD 4x20
LCDI2C_LINE1 SET     0x00            ; endereco da linha 1
LCDI2C_LINE2 SET     0x40            ; endereco da linha 2
LCDI2C_LINE3 SET     0x14            ; endereco da linha 3
LCDI2C_LINE4 SET     0x54            ; endereco da linha 4
           ENDIF


; -----------------------------------------------------------------------------------
; lcdConfig 
;
; Funcao de configuracao do LCD I2C.
;
; -----------------------------------------------------------------------------------
           goto    lcdConfExit         ; pula o codigo abaixo na primeira vez
lcdConfig  movlw   LCDI2C_ADDRESS      ; endereco I2C do LCD
           movwf   lcdaddress          ; salva o endereco
           clrf    lcdflags            ; limpa os flags do servico

           I2C_SHUTDOWN                            ; isto e' do modulo i2cmssp_services.inc
           I2C_CONFIG _FOSC, _100KHZ, NO_SLEWRATE  ; isto e' do modulo i2cmssp_services.inc
           call    lcdOpen             ; START COND + WRITE ADDRESS
           call    lcdWakeUp           ; acorda o LCD

       ; --------------------------------------------------------------------------
       ; Configuracao das caracteristicas do LCD 
       ; Comando FUNCTION --> L    L    H    DL   N    F    X    X        
       ;                      DL -> H=8 bits interface     L=4 bits interface
       ;                      N -> H=display de 2 linhas   L=dsiplay de 1 linha
       ;                      F -> H=matriz 5X10           L=matriz 5X7
       ; --------------------------------------------------------------------------
       IF LCDI2C_MAXLINES==.2 
           movlw   0x28                ; Function CMD (0x20), 4 bits, 2 linhas, 5x7
       ENDIF

       IF LCDI2C_MAXLINES==.4 
           movlw   0x20                ; Function CMD (0x20), 4 bits, 4 linhas, 5x7
       ENDIF

       IF LCDI2C_MODO==.4
           andlw   0xEF                ; DL=0           
       ENDIF

       IF LCDI2C_MODO==.8
           iorlw   0x10                ; DL=1           
       ENDIF

       IF  LCDI2C_MATRIZ==.7
           andlw   0xFB                ; matrix 5X7
       ENDIF

       IF  LCDI2C_MATRIZ==.10
           iorlw   0x04                ; matrix 5X10
       ENDIF

           call    lcdSendCmd
           movlw   .5
           call    lcdStrobeTme

       ; --------------------------------------------------------------------------
       ; Configuracao das opcoes iniciais do display 
       ;
       ; O display pode ser iniciado de varias maneiras:
       ; Display On/Off
       ; Cursor On/Off
       ; Cursor Piscante/Nao piscante
       ;
       ; Comando DISPLAY ---> L    L    L    L    H    D    C    B 
       ;                      D -> H=display on            L=display off
       ;                      C -> H=cursor on             L=cursor off
       ;                      B -> H=blinking on           L=blinking off
       ; --------------------------------------------------------------------------
           movlw   0x0C                ; Display  CMD (0x0C) --> Display On
       IF  LCDI2C_CURSOR==CURSOR_ON
           iorlw   0x02                ; cursor On
       ENDIF
 
       IF  LCDI2C_CURSOR==CURSOR_OFF
           andlw   0xFD                ; cursor Off
       ENDIF

       IF  LCDI2C_CURTYPE==BLINK
           iorlw   0x01                ; cursor piscante
       ENDIF

       IF  LCDI2C_CURTYPE==NOBLINK
           andlw   0xFE                ; cursor nao piscante
       ENDIF

           movwf   lcddisplay          ; salva configuracao do display
           call    lcdSendCmd
           movlw   .5                  ; 5ms
           call    lcdStrobeTme

       ; --------------------------------------------------------------------------
       ; ENTRY MODE --> L   L    L    L    L    L    L    H    ID   SH
       ;                ID -> H=increase              L=decrease
       ;                SH -> H=display shifted       L=display not shifted
       ; --------------------------------------------------------------------------
           movlw   0x06           ; Entry mode CMD (0x04), Inc Display, Not Shifted
           call    lcdSendCmd
       ; --------------------------------------------------------------------------
           movlw   .250
           call    delay_ms
           call    lcdClose            ; STOP COND
           retlw   .0
lcdConfExit
   endif   ;  ifndef __LCDI2C__
           call    lcdConfig
                   ENDM









; -----------------------------------------------------------------------------------
; SETI2C_CURSOR                  
;
; Posiciona o cursor    SETI2C_CURSOR  linha, coluna, tipo
;                     
; Onde:
; linha ................... numero da linha
; coluna .................. numero da coluna
; tipo .................... "i" --> linha e colunas estao inline na macro
;                           "r" ou "R" --> linha e coluna estao em registradores
; EXEMPLO:
;              SETI2C_CURSOR  .2, .1 "i"
;
; ATENCAO: Esta macro usa as funcoes 'lcdFixLine' e 'lcdSetCursor' que sao 
;          definidas dentro do corpo desta macro. Isto deve permanecer como esta
;          porque essas funcoes devem ser geradas pelo compilador somente quando 
;          esta macro for usada pela primeira vez.
; -----------------------------------------------------------------------------------
SETI2C_CURSOR MACRO linha, coluna, tipo
  IFNDEF __SETCURSOR__
#define  __SETCURSOR__  .1

          goto   _eoSetCursor              ; salta as funcoes na primeira vez
; -----------------------------------------------------------------------------
; lcdSetCursor - Posiciona o cursor 
;
; ENTRADA: lcdi2cr1 contem o numero da linha (menos 1) ajustado pela 
;          macro SETI2C_CURSOR.
;          lcdi2cr2 contem o numero da coluna (menos 1) ajustado pela 
;          macro SETI2C_CURSOR.
; 
; SAIDA  : LCD posicionado
;
; -----------------------------------------------------------------------------
lcdSetCursor call    lcdFixLine                  ; busca o endereco da linha em W
             addwf   lcdi2cr2, w                 ; adiciona o numero da coluna
             iorlw   0x80                        ; mistura endereco calculado com o comando
             movwf   lcdcmd                      ; guarda o comando para mudar de linha
             call    lcdOpen                     ; START
             movf    lcdcmd, w                   ; pega o comando              
             call    lcdSendCmd                  ; envia o comando
             call    lcdClose                    ; STOP
             return

; ----------------------------------------------------------------------------
; Retorna o endereco de uma linha do LCD  
;
; Entrada: lcdi2cr1 contem o numero da linha (menos 1) ajustado pela macro
; SET_CURSOR.
;
; Saida  : W contem o endereco da linha
;
; ----------------------------------------------------------------------------
lcdFixLine
          movf   lcdi2cr1, w                     ; pega o numero da linha
          xorlw  0x00                            ; verifica se e' 1
          btfsc  STATUS, Z                       ; linha 1???
          retlw  LCDI2C_LINE1                    ; SIM --> retorna endereco da linha 1

          movf   lcdi2cr1, w                     ; pega o numero da linha
          xorlw  0x01                            ; verifica se e' 2
          btfsc  STATUS, Z                       ; linha 2???
          retlw  LCDI2C_LINE2                    ; SIM --> retorna endereco da linha 2

 if LCDI2C_MAXLINES==.4
          movf   lcdi2cr1, w                     ; pega o numero da linha
          xorlw  .2                              ; verifica se e' 3
          btfsc  STATUS, Z                       ; linha 3???
          retlw  LCDI2C_LINE3                    ; SIM --> retorna endereco da linha 3

          movf   lcdi2cr1, w                     ; pega o numero da linha
          xorlw  .3                              ; verifica se e' 4
          btfsc  STATUS, Z                       ; linha 4???
          retlw  LCDI2C_LINE4                    ; SIM --> retorna endereco da linha 3
  endif
          retlw  LCDI2C_LINE1                    ; se nao for linha 1, 2, 3 ou 4, assume linha 1
_eoSetCursor
    ENDIF    ;;;  ifndef __SETCURSOR__

       ; ----------------------------------------------------------------------
       ; Da segunda vez em diante, somente esta segmento sera gerado pela macro
       ; ----------------------------------------------------------------------
       IF  (tipo=="R") || (tipo=="r")
           movf   linha, w                   ; aqui, 'linha' e' um registrador
           movwf  lcdi2cr1                   ; salva a linha
           decf   lcdi2cr1,f                 ; linha = linha - 1
           movf   coluna, w                  ; pega coluna
           movwf  lcdi2cr2                   ; salva coluna
           decf   lcdi2cr2,f                 ; coluna = coluna - 1
       ELSE
           movlw  linha                      ; aqui, 'linha' e' uma constante
           movwf  lcdi2cr1                   ; salva a linha
           decf   lcdi2cr1,f                 ; linha = linha - 1
           movlw  coluna                     ; pega coluna
           movwf  lcdi2cr2                   ; salva coluna
           decf   lcdi2cr2,f                 ; coluna = coluna - 1
       ENDIF
           call   lcdSetCursor               ; vai posicionar o cursor
              ENDM             ;;;FIM  DA MACRO 'SETI2C_CURSOR' 



; -----------------------------------------------------------------------------
; LCDI2C_WRITE - Envia a string inline na macro para o LCD 
;
; Exemplo:    LCDI2C_WRITE  "Isto e uma mensagem" 
; 
; -----------------------------------------------------------------------------
LCDI2C_WRITE   MACRO   st 
  IFNDEF __LCDI2CWRITE__
#define  __LCDI2CWRITE__  .1
               goto    _lcdi2cWrExit    ; salta o codigo na primeira vez
; ----------------------------------------------------------------------------
; lcdSendStr - Envia uma string terminada por nulo (0x00). 
;
; A string deve ser definida por 'DW' logo apos a chamada da funcao 'lcdSendStr'.
;
; Exemplo:     call    lcdSendStr
;              DW      "Ola!!!",0
;
; A macro LCDI2C_WRITE pode ser usada para fazer a chamada a esta funcao.
;
; ----------------------------------------------------------------------------
lcdSendStr     movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW
               call   _lcdi2c_str_c              ; vai imprimir
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               return                            ; fim

_lcdi2c_str_c  call    lcdOpen
_lcdi2cc       tblrd   *+                        ; le um caracter na lista
	           movf    TABLAT,w                  ; pega o caracter lido
	           bz      _lcdi2cftr                ; termina se for nulo
               call    lcdSendChar               ; senao, imprime o caracter
               bra     _lcdi2cc                  ; continua ate encontrar um nulo
_lcdi2cftr     call    lcdClose
               return                            ; fim
_lcdi2cWrExit
     ENDIF     ;;; IFNDEF __LCDI2CWRITE__
               call    lcdSendStr
               dw      st,0x00
               ENDM

; --------------------------------------------------------------------------------
; LCDI2C_PUT MACRO 
; Envia o caracter em W
; --------------------------------------------------------------------------------
LCDI2C_PUT MACRO
           movwf   lcdi2cr1                     ; salva o caracter
           call    lcdOpen
           movf    lcdi2cr1, w
           call    lcdSendChar
           call    lcdClose
             ENDM

; --------------------------------------------------------------------------------
; LCDI2C_PUTCHAR MACRO 
; Envia o caracter inline na macro
; --------------------------------------------------------------------------------
LCDI2C_PUTCHAR MACRO ch
           movlw   ch
           movwf   lcdi2cr1                 ; salva o caracter
           call    lcdOpen
           movf    lcdi2cr1, w
           call    lcdSendChar
           call    lcdClose
             ENDM

; --------------------------------------------------------------------------------
; LCDI2C_PRINT 
;
; Mostra no LCD uma 'string' de tamanho 'len' residente na RAM.
; --------------------------------------------------------------------------------
LCDI2C_PRINT MACRO string, len
             lfsr  FSR0, string
             movlw len
             call  lcdPrint
             ENDM

; --------------------------------------------------------------------------------
; lcdWakeUp 
;
; Acorda o LCD enviando tres vezes a sequencia 0x03.
; Esta funcao sera usada pela rotina 'lcdConfig' para sinalizar ao LCD que deve
; iniciar as funcoes de configuracao.
; Esta e' uma funcao interna que nao deve ser usada no programa.
; --------------------------------------------------------------------------------
lcdWakeUp  movlw   0x03                ; <P4-P7> = 0x03   <P0-P3> = RS=0, RW=0, LIGHT=0  
           call    lcdSendCmd
           movlw   .15
           call    lcdStrobeTme
           movlw   0x03                ; <P4-P7> = 0x03   <P0-P3> = RS=0, RW=0, LIGHT=0 
           call    lcdSendCmd
           movlw   .5
           call    lcdStrobeTme
           movlw   0x03                ; <P4-P7> = 0x03   <P0-P3> = RS=0, RW=0, LIGHT=0 
           call    lcdSendCmd
           movlw   .5
           call    lcdStrobeTme
           movlw   0x02                ; <P4-P7> = 0x02   <P0-P3> = RS=0, RW=0, LIGHT=0   
           call    lcdSendCmd
           movlw   .5
           call    lcdStrobeTme
           return

; --------------------------------------------------------------------------------
; lcdOpen 
; 
; Envia condicao de START e o endereco de WRITE ao LCD I2C.
;
; Entrada: O registrador 'lcdaddress' contem o endereco real do LCD I2C (default 0x27)
;
; Saida  : W=0 ----> ok
;          W !=0 --> erro
; --------------------------------------------------------------------------------
lcdOpen    movf    lcdaddress, w         ; carrega o endereco I2C do LCD
           movwf   lcdbuf                ; lcdbuf = 0010.0111    0x27
           rlcf    lcdbuf, f             ; lcdbuf = 0100.111x
           bcf     lcdbuf, 0             ; lcdbuf = 0100.1110    0x4E (write address)
           movff   lcdbuf, lcdwradd      ; salva endereco de write
           call    i2cStart              ; envia START CONDITION
           xorlw   0x00                  ; verifica o retorno
           btfss   STATUS, Z             ; funcionou???
           retlw   .1                    ; NAO --> retorna com erro em W
           movf    lcdwradd, w           ; endereco de WRITE do dispositivo
           call    i2cPut                ; envia endereco de WRITE
           call    i2cWaitAck            ; espera completar
           bsf     lcdflags, lightOn     ; acende o back light
           retlw   .0                    ; retorna OK

; --------------------------------------------------------------------------------
; lcdClose 
; Envia condicao de STOP e coloca o Bus em IDLE State.
; --------------------------------------------------------------------------------
lcdClose   call    i2cStop
           call    i2cIdle
           return

; ----------------------------------------------------------------------------
; lcdSendChar 
;
; ALIASES: lcdSendW
;
; Envia um caracter ao LCD (RS=1).
; O caracter sera enviado em duas etapas: primeiro o MSB e depois o LSB.
; A opcao de back-light sera inserida no pela funcao lcdSetLingh.
;
; Entrada: W contem o caracter completo de 8 bits.
;
; ----------------------------------------------------------------------------
lcdSendChar
lcdSendW
           movwf   lcdcmd        ; W = MMMM.LLLL
           andlw   0xF0          ; W = MMMM.0000
           iorlw   0x01          ; LIGHT=0, EN=0, RW=0, RS=1 
           call    lcdSetLight   ; liga/desliga back light no bit 3 do comando
           call    lcdSendByte   ; envia o MSB
           swapf   lcdcmd, w     ; W = LLLL.MMMM
           andlw   0xF0          ; W = LLLL.0000 
           iorlw   0x01          ; LIGHT=0, EN=0, RW=0, RS=1 
           call    lcdSetLight   ; liga/desliga back light no bit 3 do comando
           call    lcdSendByte   ; envia o LSB
           return

; ----------------------------------------------------------------------------
; lcdSendCMD 
;
; Envia um comando ao LCD (RS=0).
; O comando sera enviado em duas etapas: primeiro o MSB e depois o LSB.
; A opcao de back-light sera inserida no comando pela funcao lcdSetLingh.
;
; Entrada: W contem o comando completo de 8 bits.
;
; ----------------------------------------------------------------------------
lcdSendCmd movwf   lcdcmd                        ; W = MMMM.LLLL
           andlw   0xF0                          ; W = MMMM.0000 (RS=0, RW=0, EN=0, LIGHT=0) 
           call    lcdSetLight                   ; liga/desliga back light no bit 3 do comando
           call    lcdSendByte                   ; envia o MSB
           swapf   lcdcmd, w                     ; W = LLLL.MMMM
           andlw   0xF0                          ; W = LLLL.0000 (RS=0, RW=0, EN=0, LIGHT=0) 
           call    lcdSetLight                   ; liga/desliga back light no bit 3 do comando
           call    lcdSendByte                   ; envia o LSB
           return


; ----------------------------------------------------------------------------
; lcdSendByte 
;
; Esta funcao move um byte para as portas P0 a P7 do driver PCF8574T que,
; por sua vez envia 4 bits de dados ao LCD. 
; Os pinos P0 a P3 do PCF8574T estao conectados aos pinos RS, RW, EN e ANODE 
; do LCD, respectivamente. 
; Os pinos P4 a P7 estao conectados aos pinos D4 a D7 do LCD, respectivamente.
; Para o LCD receber o dado, sera simulado um strobe no pino P2/EN.
;
; Entrada: W contem o byte.
; Saida  : Byte movido ao PC.
;
; NOTAS:
; 1 - Os tempos sao criticos. O valor 80us foi obtido empiricamente. 
; ----------------------------------------------------------------------------
lcdSendByte
           movwf   lcdbuf
           andlw   0xFB               ; EN=0 (strobe p/ LCD) 
           call    i2cPut
           call    i2cWaitAck
           movlw   .80
           call    delay_us 
           movf    lcdbuf, w
           iorlw   0x04               ; EN=1 (strobe p/ LCD) 
           call    i2cPut
           call    i2cWaitAck
           movlw   .80
           call    delay_us 
           movf    lcdbuf, w
           andlw   0xFB               ; EN=0 (strobe p/ LCD) 
           call    i2cPut
           call    i2cWaitAck
           movlw   .80
           call    delay_us 
           return

; ----------------------------------------------------------------------------
; lcdCursorHome 
; Coloca o cursor na primeira posicao do LCD.
; ----------------------------------------------------------------------------
lcdCursorHome
           call    lcdOpen
           movlw   0x02                ; Cursor Home
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme 
           return

; ----------------------------------------------------------------------------
; lcdCursorOn 
; Torna o cursor visivel.
; ----------------------------------------------------------------------------
lcdCursorOn
           call    lcdOpen
           movf    lcddisplay, w
           iorlw   0x02
           movwf   lcddisplay
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme 
           return   

; ----------------------------------------------------------------------------
; lcdCursorOff 
; Torna o cursor invisivel.
; Se o cursor esta' piscante, primeiro torna-lo nao-piscante (lcdCursorNoBlink)
; para depois faze-lo invisivel.
; ----------------------------------------------------------------------------
lcdCursorOff
           call    lcdOpen
           movf    lcddisplay, w
           andlw   0xFD
           movwf   lcddisplay
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme 
           return   

; ----------------------------------------------------------------------------
; lcdCursorBlink 
; Torna o cursor piscante.
; ----------------------------------------------------------------------------
lcdCursorBlink
           call    lcdOpen
           movf    lcddisplay, w
           iorlw   0x01
           movwf   lcddisplay
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme 
           return

; ----------------------------------------------------------------------------
; lcdCursorNoBlink 
; Torna o cursor nao piscante. 
; ----------------------------------------------------------------------------
lcdCursorNoBlink
           call    lcdOpen
           movf    lcddisplay, w
           andlw   0xFE
           movwf   lcddisplay
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme 
           return

; ----------------------------------------------------------------------------
; lcdClear -  Limpa o LCD 
;
; Esta e' a mais demorada das funcoes. Apesar do envio do comando para o 
; PCF8574T ser muito rapido, o LCD necessita de muito tempo para processar o 
; comando.
; ----------------------------------------------------------------------------
lcdClear   call    lcdOpen
           movlw   0x01                ; Clear
           call    lcdSendCmd
           movlw   0x80                ; linha 1, coluna 1
           call    lcdSendCmd
           call    lcdClose
           movlw   .5                 ; espera o LCD completar a operacao
           call    lcdStrobeTme
           return

; ----------------------------------------------------------------------------
; lcdBackOn 
;
; Esta funcao acende o back-light do LCD e ligando o bit indicador de LED 
; aceso --> lcdflags<lightOn>=1.
;
; Aconselha-se limpar o LCD (lcdClear) apos o uso desta funcao.
; ----------------------------------------------------------------------------
lcdBackOn
lcdLightOn
           call    lcdOpen
           movlw   0x08
           call    lcdSendByte
           bsf     lcdflags, lightOn
           call    lcdClose
           return

; ----------------------------------------------------------------------------
; lcdBackOff 
;
; Esta funcao apaga o back-light do LCD e desligando o bit indicador de LED 
; aceso --> lcdflags<lightOn>=0.
; ----------------------------------------------------------------------------
lcdBackOff
lcdLightOff
           call    lcdOpen
           movlw   0x00
           call    lcdSendByte
           bcf     lcdflags, lightOn
           call    lcdClose
           return

; ----------------------------------------------------------------------------
; lcdDisplayOff 
; Desabilita o display
; ----------------------------------------------------------------------------
lcdDisplayOff
           call    lcdOpen
           movlw   0x08
           call    lcdSendCmd
           call    lcdClose
           movlw   .5
           call    lcdStrobeTme
           return

; ----------------------------------------------------------------------------
; lcdDisplayOn 
; Habilita o display
; ----------------------------------------------------------------------------
lcdDisplayOn
           call    lcdOpen
           movf    lcddisplay, w
           call    lcdSendCmd
           call    lcdClose
           movlw   .5                  ; 5ms
           call    lcdStrobeTme
           return

; ----------------------------------------------------------------------------
; lcdStrobeTme 
;
; Esta funcao faz retardos de 1ms a 10ms pois o minimo admitido pela funcao
; 'delay_ms' e' 10ms.
;
; Esta e' uma funcao interna. Nao usar na programacao normal.
;
; Entrada: W contem o numero de milisegundos
;
; ----------------------------------------------------------------------------
lcdStrobeTme
           movwf   lcdstrtime
           rlcf    lcdstrtime, f
           rlcf    lcdstrtime, f
           movlw   0xFC                ; tira sugeira inserida pelo Carry
           andwf   lcdstrtime, f
lcdStrobeT movlw   .250
           call    delay_us
           decfsz  lcdstrtime, f
           goto    lcdStrobeT
           return

; ----------------------------------------------------------------------------
; lcdSetLight 
;
; Esta funcao liga ou desliga o bit de controle de Back-Light do LCD no 
; comando/dado a ser enviado de acordo com o bit lcdflags<lightOn> 
; estabelecido pelas ultimas funcoes 'lcdOpen' ou 'lcdBackOn' ou 
; lcdBack_Off'.
;
; Nao usar esta funcao para programar. Usar lcdLightOn e lcdLightOff.
; ----------------------------------------------------------------------------
lcdSetLight
           andlw   0xF7               ; apaga back-light no buffer
           btfsc   lcdflags, lightOn  ; deseja acender back-light?
           iorlw   0x08               ; SIM --> acende
           return

; ----------------------------------------------------------------------------
; lcdSetCgram 
; Estabelece o endereco da CGRAM.
; Entrada: W conten o endereco
; ----------------------------------------------------------------------------
lcdSetCgram
           iorlw   0x40               ; mistura comando com endereco
           movwf   lcdcmd
           call    lcdOpen
           movf    lcdcmd, w
           call    lcdSendCmd
           call    lcdClose
          ; movlw   .5
           ;call    lcdStrobeTme
           return

; ----------------------------------------------------------------------------
; lcdShiftRight 
; Desloca o display N posicoes para a direita.
;
; Entrada: W contem o numero de posicoes.
; ----------------------------------------------------------------------------
lcdShiftRight movwf   lcdi2cr1
              call    lcdOpen
lcdSRight     movlw   0x1C                 ; comando SHIFT RIGHT
              call    lcdSendCmd
              decfsz  lcdi2cr1, f
              goto    lcdSRight
              call    lcdClose
              movlw   .100
              call    delay_us
              return

; ----------------------------------------------------------------------------
; lcdShiftLeft 
; Desloca o display N posicoes para a esquerda.
;
; Entrada: W contem o numero de posicoes.
; ----------------------------------------------------------------------------
lcdShiftLeft  movwf   lcdi2cr1
              call    lcdOpen
lcdSLeft      movlw   0x18                 ; comando SHIFT LEFT
              call    lcdSendCmd
              decfsz  lcdi2cr1, f
              goto    lcdSLeft
              call    lcdClose
              movlw   .100
              call    delay_us
              return

; ----------------------------------------------------------------------------
; lcdPrint 
; Envia N caracteres de uma string na RAM.
;
; Entrada: W contem o tamanho da string.
;          FSR0 aponta inicio da string.
;
; ----------------------------------------------------------------------------
lcdPrint      movwf   lcdi2cr1
              call    lcdOpen
_lcdi2c_print movf    POSTINC0, w
              call    lcdSendCmd
              decfsz  lcdi2cr1, f
              goto    _lcdi2c_print
              return

; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
; ----------------------------------------------------------------------------
