; ---------------------------------------------------------------------------------------------------
; LCD I2C *** TESTE DO DRIVER PARA LCD I2C
;
; Teste do driver 'lcdi2c_services.inc' para LCD I2C usando modulo MSSP.
;
; Este projeto foi testado em microcontroladores PIC18F4520, PIC18F4620 e PIC 18F4550 e funcionou
; em todos os casos. A unica diferenca entre eles e' a posicao dos pinos SCL e SDA. Ver data sheet.
;
; O LCD I2C possui um LCD modelo 44780 (1620) e um chip PCF8574T cujos pinos fazem interface 
; entre os pinos I2C do microcontrolador (RC3/SCL e RC4/SDA) e os pinos do LCD.
;
; Ver esquema de conexoes abaixo: 
; 
; PIC18F4520           PCF8574T        LCD 
; ------------         -------------   ------- 
; (18) RC3/SCL ------> (15) SDA        
; (23) RC4/SDA <-----> (14) SCL
;                      (04) P0 ------> (04) RS
;                      (05) P1 ------> (05) RW  
;                      (06) P2 ------> (06) EN  
;                      (07) P3 ------> (03) VE (ao transistor do contraste)  
;                      (09) P4 ------> (11) D4   
;                      (10) P5 ------> (12) D5  
;                      (11) P6 ------> (13) D6
;                      (12) P7 ------> (14) D7
;
; NOTAS:
; 1 - O endereco I2C de 7 bits do chip PCF8574T e' fixado na fabrica em 0x27 mas outros modelos podem
;     ter enderecos que variam entre 0x20 e 0x27. O endereco de WRITE de 8 bits utilizado sera 0x4E.
;
; 2 - O pino P3 do PCF8574T esta conectado internamente a base de um trasistor NPN e ao pino 3 (VE)
;     do LCD. Este pino e' usado para acender/apagar o back-light do LCD.
;
; 3 - Usar resistores de 4K7 no Bus I2C.
;
; ---------------------------------------------------------------------------------------------------
; HARDWARE
; Microcontrolador .................. PIC18F4520
; Fosc .............................. Externo de 20MHZ
; RC3 ............................... I2C SCL  ---> ao SCL do LCD com resistor pullup 4K7
; RC4 ............................... I2C SDA  ---> ao SDA do LCD com resistor pullup 4K7
; ---------------------------------------------------------------------------------------------------
;
	list      p=18F4520           ; list directive to define processor
	#include <p18F4520.inc>       ; processor specific variable definitions
	errorlevel  -302              ; suppress message 302 from list file
; ---------------------------------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; ---------------------------------------------------------------------------------------------
	CONFIG MCLRE  = ON
    CONFIG OSC    = HS         ; oscilador a cristal externo

; ---------------------------------------------------------------------------------------------
              #include  "Includes\familias.inc"
;
CLOCK      equ     .20000000            ; Fosc
;
           UDATA   0x80
; ---------------------------------------------------------------------------
; Registradores de controle usados pela funcao de geracao de fontes grandes
; ---------------------------------------------------------------------------
_rgcharct   res   .1
_rglinha    res   .1
_rgcoluna   res   .1
_rgptrl     res   .1

; ---------------------------------------------------------------------------
; Registradores de controle usados pelo driver 'lcdi2c_services.inc'
; ---------------------------------------------------------------------------
_FOSC         equ     CLOCK            ; Fosc 
_100KHZ       equ     .100000          ; taxa de transmissao do I2C 100KHz
lcdi2cr1      res     .1
lcdi2cr2      res     .1
lcdaddress    res     .1
lcdwradd      res     .1
lcdcmd        res     .1
lcdbuf        res     .1
lcdstrtime    res     .1
lcddisplay    res     .1
lcdflags      res     .1
lightOn       equ     .7
i2c_buff      res     .1
i2c_timeoutL  res     .1
i2c_timeoutH  res     .1
temp_temp     res     .1
delay_temp    res     .1
         
; ---------------------------------------------------------------------------
; Registradores de trabalho
; ---------------------------------------------------------------------------
R4            res     .1     ; registrador de trabalho
R5            res     .1     ; registrador de trabalho
R6            res     .1     ; registrador de trabalho
R7            res     .1     ; registrador de trabalho
R12           res     .1     ; registrador de trabalho
R13           res     .1     ; registrador de trabalho

;
; ---------------------------------------------------------------------------------------------
;
pwron      CODE    0x0000
           goto    inicio
;
intrf      CODE    0x008
           retfie

; ---------------------------------------------------------------------------------------------
;
MAIN       CODE

           #include "Includes\lcdi2c_services.inc"           ; CONTROLADOR DO LCD

; ---------------------------------------------------------------------------------------------
;
inicio     movlw   0x0F 
           movwf   ADCON1                        ; todas as portas sao I/O digital
           clrf    LATA
           clrf    LATB
           clrf    LATC
           clrf    LATD
           clrf    LATE
           clrf    TRISA
           clrf    TRISB
           clrf    TRISC
           clrf    TRISD

           ; CONFIGURACAO DO LCD I2C
           LCDI2C_CONFIG  0x27, .2, .16, .4, .7, CURSOR_ON, NOBLINK
           TEMPO   .1                  ; espera tudo estabilizar  

           ; Teste da funcao LCD_WRITE
teste      call   lcdClear
           SETI2C_CURSOR .1, .1, "i"
           LCDI2C_WRITE "HPSPIN * LCD I2C"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "TESTANDO 18F4520"
           TEMPO   .2
           call    vaivem
           TEMPO   .2

           ; Teste do cursor
cursoronof call    lcdClear
           LCDI2C_WRITE "CURSOR LIGADO"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "-->     PISCANDO"
           SETI2C_CURSOR .2, .4, "i"
           call    lcdCursorOn
           call    lcdCursorBlink
           TEMPO   .3

           call    lcdClear
           LCDI2C_WRITE "CURSOR LIGADO"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "-->     ESTATICO"
           SETI2C_CURSOR .2, .4, "i"
           call    lcdCursorNoBlink
           call    lcdCursorOn
           TEMPO   .3

           call    lcdClear
           LCDI2C_WRITE "CURSOR DESLIGADO"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "-->   SEM CURSOR"
           SETI2C_CURSOR .2, .4, "i"
           call    lcdCursorOff
           TEMPO   .3

           call    lcdClear
           LCDI2C_WRITE " <-- CURSOR HOME"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "CURSOR PISCANDO"
           call    lcdCursorHome
           call    lcdCursorOn
           call    lcdCursorBlink
           TEMPO   .3

           call    lcdClear
           LCDI2C_WRITE " <-- CURSOR HOME"
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "CURSOR ESTATICO"
           call    lcdCursorHome
           call    lcdCursorOn
           call    lcdCursorNoBlink
           TEMPO   .3


           ; Teste de habilitaca e desabilitacao do display
           call    lcdClear
           LCDI2C_WRITE "DESABILITANDO..."
           TEMPO   .1
           call    lcdDisplayOff
           TEMPO   .3
           call    lcdClear
           LCDI2C_WRITE "LCD HABILITADO"
           call    lcdDisplayOn
           TEMPO   .3


           ; Teste das funcoes da apagar e acender o back-light
disponof   movlw   .2
           movwf   R5
disploop   call    lcdClear
           LCDI2C_WRITE "LCD VAI APAGAR"
           TEMPO   .1
           call    lcdBackOff
           TEMPO   .2
 
           call    lcdBackOn
           call    lcdClear
           LCDI2C_WRITE "LCD ACESO"
           TEMPO   .2
           decfsz  R5, f
           goto    disploop
       

           ; Teste do cursor com posicoes passadas nos registradores
           call   lcdClear
           movlw   .1
           movwf   R5                  ; linha
           movwf   R6                  ; coluna
           movlw   .16
           movwf   R7                  ; numero de caracteres
           movlw   "A"
           movwf   R4                  ; caracter inicial
printb     SETI2C_CURSOR R5, R6, "r"
           call    lcdOpen
           movf    R4, w
           incf    R4, f               ; proximo caracter
           call    lcdSendChar
           call    lcdClose
           incf    R6, f               ; col = col + 1
           decfsz  R7, f
           goto    printb
           TEMPO   .1
           movlw   .2
           movwf   R5                  ; linha
           movlw   .1
           movwf   R6                  ; coluna
           movlw   .16
           movwf   R7                  ; numero de caracteres
           movlw   "Z"
           movwf   R4                  ; caracter inicial
printc     SETI2C_CURSOR R5, R6, "r"
           call    lcdOpen
           movf    R4, w
           decf    R4, f               ; caracter anterior
           call    lcdSendChar
           call    lcdClose
           incf    R6, f               ; col = col + 1
           decfsz  R7, f
           goto    printc
           TEMPO   .1
           call    vaivem
           TEMPO   .1    

           ; Teste de de geracao de caracteres na CGRAM
cgramtest  movlw   0x00                ; endereco na CGRAM
           call    lcdSetCgram         ; posiciona a CGRAM
           call    lcdOpen
           movlw   0x1F
           call    lcdSendChar
           movlw   0x11
           call    lcdSendChar
           movlw   0x11
           call    lcdSendChar
           movlw   0x15
           call    lcdSendChar
           movlw   0x15
           call    lcdSendChar
           movlw   0x11
           call    lcdSendChar
           movlw   0x11
           call    lcdSendChar
           movlw   0x1F
           call    lcdSendChar
           call    lcdClose
           call    lcdClear
           LCDI2C_WRITE "CARACTER CGRAM"
           SETI2C_CURSOR .2 , .1, "i"
           movlw   .16
           movwf   R6
cgshow     LCDI2C_PUTCHAR 0x00
           decfsz  R6, f     
           goto    cgshow
           TEMPO   .2


           ; Teste de fontes grandes
bigchars   call    lcdClear            ; limpa o LCD
           LCDI2C_WRITE "CONTAGEM   "
           SETI2C_CURSOR .2, .1, "i"
           LCDI2C_WRITE "REGRESSIVA "
           call    font_load           ; carrega a fonte na CGRAM
bignext    movlw   .9                  ; inicia contagem em 9
           movwf   R13                 ; contagem regressiva de 9 a 1
bigchct    movlw   .12                  ; numero da coluna
           movwf   _rgcoluna           ; posiciona na coluna 7
           movf    R13, w              ; pega um digito da contagem
           call    font_put            ; mostra na coluna 7
           TEMPO   .1                  ; tempo para visualizacao
           decfsz  R13, f              ; contagem
           goto    bigchct             ; vai mosttrar o proximo digito


           goto    teste
 
; ------------------------------------------------------------------------
; Desloca todo o display para a direita e retorna para a posicao original
; ------------------------------------------------------------------------
vaivem     movlw   .1
           movwf   R6
palapaca   movlw   .15
           movwf   R7
paDireita  movlw   .1
           call    lcdShiftRight
           movlw   .90
           call    delay_ms
           decfsz  R7, f
           goto    paDireita
           movlw   .15
           movwf   R7
paEsquerda movlw   .1
           call    lcdShiftLeft
           movlw   .90
           call    delay_ms
           decfsz  R7, f
           goto    paEsquerda
           decfsz  R6, f
           goto    palapaca
           return

; ------------------------------------------------------------------------
; Inclusao da funcao para geracao e display de fonte numerica grande.
; Para mais informacoes, ler as instrucoes no arquivo fonte.
; ------------------------------------------------------------------------
           #include "halfsz2x3f18eep_src.inc"

           END
